/**
 * Copyright (c) 2007-2009 Eric Torreborre <etorreborre@yahoo.com>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated
 * documentation files (the "Software"), to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all copies or substantial portions of
 * the Software. Neither the name of specs nor the names of its contributors may be used to endorse or promote
 * products derived from this software without specific prior written permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
 * CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */
package org.specs.literate
import org.eclipse.mylyn.wikitext.core.parser.MarkupParser
import org.eclipse.mylyn.wikitext.textile.core.TextileLanguage
import org.specs.util.ExtendedString._

trait Textile extends TextileFormatting with TextileWiki
class TextileFormatter extends TextileFormatting
trait TextileFormatting extends WikiFormatter {
  override def escapeHtml(s: String) = "=="+s+"=="
  override protected def parseToHtml(s: String) = {
    debug("before is \n" + s)
    val parser = new MarkupParser()
    parser.setMarkupLanguage(new TextileLanguage())
    val parsed = parser.parseToHtml(s)
    debug("parsed is \n" + parsed)
    val replaced = parsed.
    replace("&#8220;", "\"").
    replace("&#8221;", "\"").
    replace("&#8216;", "'").
    replace("&#8217;", "'").
    replaceGroups("(<code>((.)*)</code>)", (s: String) =>
        s.replace("<br/>", "<br></br>").
        replace("&amp;quot;", "\"")
    )
    debug("replaced is \n" + replaced)
    replaced
  }
}
